/* 
   Zapoctovy priklad do P009 - OpenGL
   autor : David Safranek <xsafran1@fi.muni.cz>
   odzkouseno na stroji - preklad na aise, spousteno na erinys04 (SGI INDY)
 
 specifikace :
  
   objekty :
     model kolotoce, koule z ruzneho materialu, umisteno na podlozce;
     svetleno stabilnim svetlem ze smeru zapadniho (pro vychozi pozici);
     pohyb objektu - ratace kolotoce a vysunovani ramen;
     pohyb kamerou - volny pohyb avatara ve scene omezeny zemi a oblohou;
  
   ovladani :
     prave tlacitko mysi - menu fci :
	   zap/vyp - animace;
	   konec aplikace;
	   navrat avatara do vychozi pozice;
     pohyb ramen kolotoce - klavesy 'A' a 'Z';
     pohyb avatara ve scene - kursorove sipky ... vyska a otaceni;
 			    - '+', '-' ... chuze tam a zpatky; 
*/

#include <stdio.h>
#include <stdarg.h>
#include <GL/glut.h>
#include <math.h>
   
#define PI 3.14
#define DEG2RAD(angle) (PI*(angle)/180.0)

void redisplay_all(void);
void CloseTheWindow();
void setProjection(void);

GLuint windowID; 
GLuint menuID;

GLuint width  = 512;	/* velikost okna */
GLuint height = 412;
enum  { PERSPEKTIVA, ROVNOBEZNE_PROMITANI } mode = PERSPEKTIVA ;

char* volba = "zadna akce";

GLboolean animace = GL_FALSE;

GLUquadricObj *cylinder;   /* dynamicke objekty glu */

double d_degree = 0;	/* uhel rotace */
double s_degree = 30;   /* uhel nakloneni speichen */
double move_y = 0.0;    
double rotate_y = 0.0;

/* nastaveni kamery */
GLfloat cam_position[3] = { 0.0, 0.0, -2.0 };
GLfloat cam_direction[3];

/* nastaveni svetla */
GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat position[] = { 3.0, 5.0, 0.0, 0.0 };
GLfloat direction[] = { 0.0, 0.0, -1.0 };
GLfloat cutoff[] = { 180.0 };
GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
GLfloat local_view[] = { 0.0 };

/* nastaveni typu materialu */
GLfloat specular[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat shininess[] = { 0.0 };
GLfloat emission[] = { 0.0, 0.0, 0.0, 1.0 };

/*GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat mat_ambient[] = { 0.7, 0.7, 0.7, 1.0 };
GLfloat mat_ambient_color[] = { 0.8, 0.8, 0.2, 1.0 };
GLfloat mat_diffuse[] = { 0.1, 0.5, 0.8, 1.0 };
GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat no_shininess[] = { 0.0 };
GLfloat low_shininess[] = { 5.0 };
GLfloat high_shininess[] = { 100.0 };
GLfloat mat_emission[] = {0.3, 0.2, 0.2, 0.0};
*/

GLvoid *font_style = GLUT_BITMAP_TIMES_ROMAN_10;


/* funkce pro zobrazeni retezce */
void setfont(char* name, int size)
{
    font_style = GLUT_BITMAP_HELVETICA_10;
    if (strcmp(name, "helvetica") == 0) {
	if (size == 12) 
	    font_style = GLUT_BITMAP_HELVETICA_12;
	else if (size == 18)
	    font_style = GLUT_BITMAP_HELVETICA_18;
    } else if (strcmp(name, "times roman") == 0) {
	font_style = GLUT_BITMAP_TIMES_ROMAN_10;
	if (size == 24)
	    font_style = GLUT_BITMAP_TIMES_ROMAN_24;
    } else if (strcmp(name, "8x13") == 0) {
	font_style = GLUT_BITMAP_8_BY_13;
    } else if (strcmp(name, "9x15") == 0) {
	font_style = GLUT_BITMAP_9_BY_15;
    }
}

void drawstr(GLuint x, GLuint y, char* format, ...)
{
    va_list args;
    char buffer[255], *s;
    
    va_start(args, format);
    vsprintf(buffer, format, args);
    va_end(args);
    
    glRasterPos2i(x, y);
    for (s = buffer; *s; s++)
	glutBitmapCharacter(font_style, *s);
}

void init(void)
{
  cam_position[0] = 0.0;
  cam_position[1] = 0.0;
  cam_position[2] = -2.0;

  /* inicializace parametru -10 -3 */
  d_degree = 0;	    /* uhel rotace kolotoce */
  s_degree = 30;    /* uhel nakloneni speichen */
  move_y = 0.5;     /* vyska kamery */
  rotate_y = 0.0;   /* rotace kamery */
}

/* umisteni kamery ve smeru aktualni rotace a posunem o krok dopredu */
void nastav_kameru(double krok)
{
  cam_direction[0] = sin(DEG2RAD(rotate_y));
  cam_direction[1] = 0.0;
  cam_direction[2] = cos(DEG2RAD(rotate_y));

  cam_position[0] += krok * cam_direction[0];
  cam_position[1] += krok * cam_direction[1];
  cam_position[2] += krok * cam_direction[2];
}

/* kresba podlozky */
void paintPlane(void) 
{
  specular[0] = 0.0; specular[1] = 0.0; specular[2] = 0.0;
  shininess[0] = 0.0;
  emission[0] = 0.0; emission[1] = 0.0; emission[2] = 0.0;
  glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, shininess);
  glMaterialfv(GL_FRONT, GL_EMISSION, emission);

  glPushMatrix();
  glRotated(90.0, 1.0, 0.0, 0.0);
  glBegin(GL_POLYGON);
	   glNormal3d(0.0, 0.0, -1.0);
	   glColor3d(0.0,0.0,1.0);
	   glVertex3d(-1.8,-1.8, 0.0);
	   glVertex3d(-1.8,1.8, 0.0);
	   glVertex3d(1.8,1.8, 0.0);
	   glVertex3d(1.8,-1.8, 0.0);
  glEnd();
  glPopMatrix();
}

/* kresba zakladniho valce kolotoce */
void paintCylinder()
{
  specular[0] = 1.0; specular[1] = 1.0; specular[2] = 1.0;
  shininess[0] = 50.0;
  emission[0] = 0.0; emission[1] = 0.0; emission[2] = 0.0;
  glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, shininess);
  glMaterialfv(GL_FRONT, GL_EMISSION, emission);

  glPushMatrix();
  glRotated(90.0, -1.0, 0.0, 0.0);
  glTranslated(0.0, 0.0, 0.1);
  glColor3d(0.0, 1.0, 0.0);
  gluCylinder(cylinder, 0.1, 0.1, 0.5, 24, 24);
  glPopMatrix();
  glPushMatrix();
  glTranslated(0.0, 0.5, 0.0);
  glutSolidSphere(0.11, 24, 24);
  glPopMatrix();
}

/* kresba strisky kolotoce */
void paintCone()
{
  specular[0] = 1.0; specular[1] = 1.0; specular[2] = 1.0;
  shininess[0] = 100.0;
  emission[0] = 0.1; emission[1] = 0.0; emission[2] = 0.0;
  glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, shininess);
  glMaterialfv(GL_FRONT, GL_EMISSION, emission);

  glPushMatrix();
  glTranslated(0.0, 0.5, 0.0);
  glRotated(90.0, -1.0, 0.0, 0.0);
  glTranslated(0.0, 0.0, 0.1);
  glColor3d(1.0, 0.0, 0.0);
  glutSolidCone(0.1, 0.25, 24, 24);
  glPopMatrix();
}

/* kresba ramena kolotoce */
void paintSpeichna(double d_angle, int matros)
{
  specular[0] = 0.0; specular[1] = 0.0; specular[2] = 0.0;
  shininess[0] = 0.0;
  emission[0] = 0.0; emission[1] = 0.0; emission[2] = 0.0;
  glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, shininess);
  glMaterialfv(GL_FRONT, GL_EMISSION, emission);

  glPushMatrix();
  glRotated(d_degree + d_angle, 0.0, 1.0, 0.0);
  glTranslated(0.11, 0.0, 0.0);
  glTranslated(0.0, 0.5, 0.0);
  glRotated(s_degree, 0.0, 0.0, -1.0); 
  glBegin(GL_LINE_LOOP);
    glColor3d(1.0, 0.0, 0.0);
    glVertex3d(0.0, -0.01, 0.01);
    glVertex3d(0.0, 0.01, 0.01);
    glVertex3d(0.6, 0.01, 0.01);
    glVertex3d(0.6, -0.01, 0.01);
  glEnd();
  glBegin(GL_LINE_LOOP);
    glColor3d(1.0, 0.0, 0.0);
    glVertex3d(0.0, -0.01, -0.01);
    glVertex3d(0.0, 0.01, -0.01);
    glVertex3d(0.6, 0.01, -0.01);
    glVertex3d(0.6, -0.01, -0.01);
  glEnd();
  glColor3d(1.0, 1.0, 0.0);
  glTranslated(0.6, 0.0, 0.0);
  glRotated(s_degree, 0.0, 0.0, 1.0);
  switch (matros) {
    case 0: {
      specular[0] = 0.0; specular[1] = 0.0; specular[2] = 0.0;
      shininess[0] = 0.0;
      emission[0] = 0.0; emission[1] = 0.0; emission[2] = 0.0;
      break;
    }
    case 1: {
      specular[0] = 1.0; specular[1] = 1.0; specular[2] = 1.0;
      shininess[0] = 10.0;
      emission[0] = 0.0; emission[1] = 0.0; emission[2] = 0.0;
      break;
    }
    case 2: {
      specular[0] = 1.0; specular[1] = 1.0; specular[2] = 1.0;
      shininess[0] = 100.0;
      emission[0] = 0.0; emission[1] = 0.0; emission[2] = 0.0;
      break;
    }
    case 3: {
      specular[0] = 1.0; specular[1] = 1.0; specular[2] = 1.0;
      shininess[0] = 50.0;
      emission[0] = 0.0; emission[1] = 0.0; emission[2] = 0.0;
      break;
    }
  }
/*glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse); */  /* pocitano ColorMaterial */
  glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, shininess);
  glMaterialfv(GL_FRONT, GL_EMISSION, emission); 
  glutSolidSphere(0.1, 12, 12);
  glPopMatrix();
}

/* vykresleni sceny */
void createScene(void)
{
  int i;

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(cam_position[0], cam_position[1] + move_y, cam_position[2],
   	    cam_position[0] + cam_direction[0],
            cam_position[1] + cam_direction[1] + move_y,
            cam_position[2] + cam_direction[2],
            0.0, 1.0, 0.0);
  glLightfv(GL_LIGHT0, GL_POSITION, position);      
  glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, direction);
  paintPlane();
  glTranslated(0.0, -0.1, 0.0);
  if (animace) d_degree = d_degree + 5; 
  /* kresba kolotoce s resenim viditelnosti - to kdyby nebyl k dispozici Z-BUFFER */
  if (d_degree + rotate_y >= 360) d_degree = 0 - rotate_y;
  if (d_degree + rotate_y >= 180) {
    paintSpeichna(180, 0);
    paintCylinder();
    paintCone();
    paintSpeichna(0, 1);
  } 
  else {
    paintSpeichna(0, 1);
    paintCylinder();
    paintCone();
    paintSpeichna(180, 0);
  }
  if (d_degree + rotate_y >= 90) {
    paintSpeichna(270, 2);
    paintCylinder();
    paintCone();
    paintSpeichna(90, 3);
  }
  else {
    paintSpeichna(90, 3);          
    paintCylinder();
    paintCone();
    paintSpeichna(270, 2);
  }
  glFlush();
   
 /*
        setfont("helvetica", 18);
        glColor3ub(255, 0, 0);
        drawstr(0.0, 0.0,"volba: %s",volba); 
 */
}

/* nastaveni parametru projekce sceny do okna aplikace */
void setProjection(void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  switch (mode) {
    case (PERSPEKTIVA): {
      gluPerspective(60.0, 1.0, 0.01, 200.0);
      break;
    }
    case (ROVNOBEZNE_PROMITANI): {
      glOrtho(-1.0,1.0,-1.0,1.0,-20.0,20.0);
      break;
    }
  }
  glMatrixMode(GL_MODELVIEW);
}

/* vykreslovaci rutina */
void main_display(void)
{
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  setProjection();
  createScene();

  glutSwapBuffers();
}

/* user interface  */
void main_keyboard(char key, int x, int y)
{
    switch (key) {
	case 'p': {
          mode = PERSPEKTIVA;
	  glutPostRedisplay();
	  break;
        }
/*        case 'r': {
          mode = ROVNOBEZNE_PROMITANI;
	  glutPostRedisplay();
	  break;
        } */
        case '+': { 			/* pohyb kamerou dopredu */
          nastav_kameru(0.1);
	  glutPostRedisplay();
	  break; 
	}
	case '-': {			/* pohyb kamerou dozadu */
          nastav_kameru(-0.1);
	  glutPostRedisplay();
	  break; 
	}
        case 'a': {			/* pohyb ramen nahoru */
          if (s_degree>-50) s_degree -= 0.3;
          glutPostRedisplay();
          break;
        }
        case 'z': {			/* pohyb ramen dolu */
          if (s_degree<30) s_degree += 0.3;
          glutPostRedisplay();
	  break;
        }
	case 'u': {			/* vratit kameru do vychozi pozice */
          init();
	  nastav_kameru(0.0);
          glutPostRedisplay();
	  break;
        }  
        case 27: {
          CloseTheWindow();  		/* escape from the scene */
	  exit(0);
	  break;
    	}
    }
}

void SpecialKeys(int key, int x, int y)
{
  switch (key) {
    case GLUT_KEY_UP: {			/* pohyb kamerou nahoru */
      if (move_y < 2.0) move_y += 0.05;
      glutPostRedisplay();
      break; 
    }
    case GLUT_KEY_DOWN: {		/* pohyb kamerou dolu */
      if (move_y > 0.1) move_y -= 0.05;
      glutPostRedisplay();
      break; 
    }
    case GLUT_KEY_LEFT: {      		/* rotace kamery vlevo */
      rotate_y += 1.0;
      nastav_kameru(0.0);
      glutPostRedisplay();
      break;
    }
    case GLUT_KEY_RIGHT:{      		/* rotace kamery vpravo */
      rotate_y -= 1.0;
      nastav_kameru(0.0);
      glutPostRedisplay();
      break;
    }
  }
}

void main_menu(int value)
{
  switch (value) {
    case 0: {				/* escape from the scene */
      CloseTheWindow();
      exit(0);
      break;
    }
    case 1: {				/* zapnuti animace */
      animace=GL_TRUE;
      break;
    }
    case 2: {				/* vypnuti animace */
      animace=GL_FALSE;
      glutPostRedisplay();
      break; 
    }
    case 3: {				/* vratit pocatecni pozici kamery */
      init();
      nastav_kameru(0.0);
      glutPostRedisplay();
      break;    
    }
    default: break;
  }
}

void main_reshape(int width,  int height) 
{
   glViewport (0, 0,width,height);
   /* setProjection(); */
}

void CloseTheWindow()
{ 
  glutDestroyMenu(menuID);
  glutDestroyWindow(windowID);
}

void world_idle(void)
{
  if (animace) glutPostRedisplay();
}

/* hlavni program */
int main(int argc, char** argv)
{
    cylinder = gluNewQuadric();   /* alokace dynamickych glu objektu */
    init();
    nastav_kameru(0.0);

    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE); 
    glutInitWindowSize(width, height);
    glutInitWindowPosition(50, 50);

    windowID = glutCreateWindow("Zapoctak do P009 'Toc-Kolotoc'");

    glEnable(GL_DEPTH_TEST);	/* uzivej Z-BUFFERU */

    glCullFace(GL_BACK);	/* radeji neukazuj zada */
    glEnable(GL_CULL_FACE); 

    glShadeModel(GL_SMOOTH);

    glDepthFunc(GL_LESS);

    /* nastaveni osvetlovaciho modelu a umisteni svetla */
    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
    glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, direction);
    glLightfv(GL_LIGHT0, GL_SPOT_CUTOFF, cutoff);
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
    glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0); 

    glEnable(GL_COLOR_MATERIAL);   

    glutReshapeFunc(main_reshape);
    glutDisplayFunc(main_display);
    glutKeyboardFunc(main_keyboard);
    glutSpecialFunc(SpecialKeys);

    menuID=glutCreateMenu(main_menu);
    glutAddMenuEntry("Konec aplikace", 0);
    glutAddMenuEntry("Roztocit kolotoc",1);
    glutAddMenuEntry("Zastavit kolotoc",2);
    glutAddMenuEntry("Vratit pocatecni stav",3);
    glutAttachMenu(GLUT_RIGHT_BUTTON|GLUT_LEFT_BUTTON); 

    glutIdleFunc(world_idle);

    glutMainLoop();

    gluDeleteQuadric(cylinder);

    return(0);
}
